@@ -44,7 +44,7 @@
 # define FLUSHCACHE
 #endif
 
-#if defined(__arm__) || defined(__arm64__) || defined(__aarch64__) || defined(_M_ARM64) || defined(__riscv)
+#if defined(__arm__) || defined(__arm64__) || defined(__aarch64__) || defined(_M_ARM64) || defined(__riscv) || defined(__loongarch64)
 # define FLUSHCACHE
 #endif
 
@@ -120,7 +120,7 @@ typedef int tputsputcchar;
 #endif
 #endif
 
-#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)
 #define NOBLOCK O_NONBLOCK
 #define LOAD_SHARED_OBJECT
 #define USE_MMAP
@@ -196,7 +196,7 @@ typedef int tputsputcchar;
 typedef char *memcpy_t;
 struct timespec;
 #ifdef __MINGW32__
-# if defined(__aarch64__)
+# if defined(__aarch64__) && !defined(PORTABLE_BYTECODE)
 #  define HAND_CODED_SETJMP_SIZE 32
 # endif
 #else
